==> ldap-manager-0.0.1-20190327.130155-5-sources.jar: com/xdja/pki/issue/PkixIssueReqBuilder.java <==
package com.xdja.pki.issue;

import com.xdja.pki.asn1.issue.ASN1CRL;
import com.xdja.pki.asn1.issue.CertStatus;
import com.xdja.pki.asn1.issue.TBSIssueRequest;
import com.xdja.pki.ldap.utils.X509Utils;
import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.ocsp.Signature;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.cert.X509CertificateHolder;
import org.bouncycastle.cert.ocsp.CertificateID;
import org.bouncycastle.operator.DefaultDigestAlgorithmIdentifierFinder;
import org.bouncycastle.operator.bc.BcDigestCalculatorProvider;
import sun.security.provider.certpath.X509CertificatePair;

import java.io.IOException;
import java.security.KeyPair;
import java.security.NoSuchProviderException;
import java.security.cert.CRLException;
import java.security.cert.CertificateException;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.List;


// TODO: 2019/2/21 签名运算后期需要调用加密机
public class PkixIssueReqBuilder extends BasicPkixIssueBuilder {

    private String hashAlgorithm;

    public PkixIssueReqBuilder(KeyPair keyPair, X509Certificate certificate, String hashAlgorithm) {
        super(keyPair, certificate);
        this.hashAlgorithm = hashAlgorithm;
    }

    public PkixIssueReq build(X509Certificate oldWithNew, X509Certificate newWithOld, X509Certificate newWithNew) throws Exception {
        if (X509Utils.verifyCert(newWithOld, this.certificate.getPublicKey())
                && X509Utils.verifyCert(oldWithNew, newWithNew.getPublicKey())
                && X509Utils.verifyCert(newWithNew, newWithNew.getPublicKey())) {
            this.certificate = newWithNew;
            return generate(TBSIssueType.UPDATE_ROOT_CERTIFICATE, new Certificate[]{X509Utils.convertCertificate(oldWithNew),
                    X509Utils.convertCertificate(newWithOld), X509Utils.convertCertificate(newWithNew)}, null, null);
        }
        throw new Exception("verify is false");
    }


    public PkixIssueReq build(X509Certificate cert) throws Exception {
        return generate(TBSIssueType.SEND_CERTIFICATE, new Certificate[]{X509Utils.convertCertificate(cert)}, null, null);
    }

    public PkixIssueReq buildSendCerts(List<X509Certificate> certs) throws Exception {
        Certificate[] certificates = new Certificate[certs.size()];
        for (int i = 0; i < certs.size(); i++) {
            certificates[i] = X509Utils.convertCertificate(certs.get(i));
        }
        return generate(TBSIssueType.SEND_CERTIFICATE, certificates, null, null);
    }

    public PkixIssueReq build(int crlSegment, X509CRL crl) throws Exception {
        return generate(TBSIssueType.SEND_CRL, null, null, new ASN1CRL[]{generateASN1CRL(crlSegment, crl)});
    }

    public PkixIssueReq buildSendCrls(List<PkixIssueCRL> list) throws Exception {
        ASN1CRL[] asn1CRLS = new ASN1CRL[list.size()];
        for (int i = 0; i < list.size(); i++) {
            asn1CRLS[i] = generateASN1CRL(list.get(i).getCrlSegment(), list.get(i).getCrl());
        }
        return generate(TBSIssueType.SEND_CRL, null, null, asn1CRLS);
    }

    public PkixIssueReq build(TBSIssueCRLReason reason, X509Certificate certificate) throws Exception {
        return generate(TBSIssueType.SEND_CERTIFICATE_STATUS, null, new CertStatus[]{generateCertStatus( reason.getType(),
                certificate)}, null);
    }

    public PkixIssueReq buildSendCertStatuses(List<PkixIssueCertStatus> list) throws Exception {
        CertStatus[] statuses = new CertStatus[list.size()];
        for (int i = 0; i < list.size(); i++) {
            statuses[i] = generateCertStatus(list.get(i).getReason().getType(), list.get(i).getCertificate());
        }
        return generate(TBSIssueType.SEND_CERTIFICATE_STATUS, null, statuses, null);
    }

    public PkixIssueReq build(X509CertificatePair pair) throws Exception {
        if (pair.getForward() == null && pair.getReverse() == null) {
            throw new Exception("can not be all null");
        }
        if (pair.getForward() == null) {
            return generate(TBSIssueType.SEND_CROSS_CERTIFICATE, new Certificate[]{
                    X509Utils.convertCertificate(pair.getReverse())
            }, null, null);
        }
        if (pair.getReverse() == null) {
            return generate(TBSIssueType.SEND_CROSS_CERTIFICATE, new Certificate[]{
                    X509Utils.convertCertificate(pair.getForward())
            }, null, null);
        }

        return generate(TBSIssueType.SEND_CROSS_CERTIFICATE, new Certificate[]{
                X509Utils.convertCertificate(pair.getForward()),
                X509Utils.convertCertificate(pair.getReverse())
        }, null, null);
    }

    private ASN1CRL generateASN1CRL(int crlSegment, X509CRL crl) throws CRLException, CertificateException, NoSuchProviderException, IOException {
        ASN1Integer stateOrProvinceNum = new ASN1Integer(0);
        ASN1Integer crlSegmentASN1 = new ASN1Integer(crlSegment);
        ASN1Integer cRLNumber = new ASN1Integer(crl.getExtensionValue(Extension.cRLNumber.getId()));
        ASN1Integer deltareCRLNumber = null;
        if (X509Utils.isDRL(crl)) {
            byte[] bytes = crl.getExtensionValue(Extension.deltaCRLIndicator.getId());
            if (bytes != null) {
                deltareCRLNumber = ASN1Integer.getInstance(DEROctetString.getInstance(bytes).getOctets());
            }
        }

        CertificateList certificateList = CertificateList.getInstance(crl.getEncoded());

        return new ASN1CRL(stateOrProvinceNum, crlSegmentASN1, cRLNumber, deltareCRLNumber, certificateList);
    }

    private CertStatus generateCertStatus(int reason, X509Certificate cert) throws Exception {
        // TODO: 2019/3/27 hash是否使用密码机 （可不使用）
        CertificateID certId = new CertificateID((new BcDigestCalculatorProvider())
                .get(new DefaultDigestAlgorithmIdentifierFinder().find(this.hashAlgorithm)),
                new X509CertificateHolder(this.certificate.getEncoded()), cert.getSerialNumber());

        ASN1GeneralizedTime beforeTime = new ASN1GeneralizedTime(cert.getNotBefore());
        ASN1GeneralizedTime endTime = new ASN1GeneralizedTime(cert.getNotAfter());
        //证书状态
        ASN1Integer asn1Status = TBSIssueCRLStatus.INVALID.encode();
        ASN1GeneralizedTime statusTime = new ASN1GeneralizedTime(new Date(System.currentTimeMillis()));
        CRLReason crlReason = CRLReason.lookup(reason);

        return new CertStatus(certId.toASN1Primitive(), beforeTime, endTime, asn1Status, statusTime, crlReason);
    }

    private PkixIssueReq generate(TBSIssueType type, Certificate[] cert, CertStatus[] certStatus, ASN1CRL[] mCRL) throws Exception {
        ASN1Integer typeASN1 = type.encode();
        ASN1OctetString transNonce = PkixIssueUtils.generateTransNonce();
        ASN1GeneralizedTime time = PkixIssueUtils.generateTime();
        ASN1Integer number;
        switch (type) {
            case UPDATE_ROOT_CERTIFICATE:
            case SEND_CERTIFICATE:
            case SEND_CROSS_CERTIFICATE:
                number = new ASN1Integer(cert.length);
                break;
            case SEND_CRL:
                number = new ASN1Integer(mCRL.length);
                break;
            case SEND_CERTIFICATE_STATUS:
                number = new ASN1Integer(certStatus.length);
                break;
            default:
                throw new Exception("unknown type " + type);
        }

        TBSIssueRequest pkixIssueInfo = new TBSIssueRequest(typeASN1, transNonce, number, time, cert, certStatus, mCRL);
        Signature signature = this.getSignature(pkixIssueInfo);
        return new PkixIssueReq(pkixIssueInfo, signature);
    }
}

==> ldap-manager-0.0.1-20190529.065003-23-sources.jar: com/xdja/pki/issue/PkixIssueReqBuilder.java <==
package com.xdja.pki.issue;

import com.xdja.pki.asn1.issue.ASN1CRL;
import com.xdja.pki.asn1.issue.CertStatus;
import com.xdja.pki.asn1.issue.TBSIssueRequest;
import com.xdja.pki.gmssl.crypto.sdf.SdfCryptoType;
import com.xdja.pki.ldap.X509Utils;
import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.ocsp.Signature;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.cert.X509CertificateHolder;
import org.bouncycastle.cert.ocsp.CertificateID;
import org.bouncycastle.operator.DefaultDigestAlgorithmIdentifierFinder;
import org.bouncycastle.operator.bc.BcDigestCalculatorProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sun.security.provider.certpath.X509CertificatePair;

import java.io.IOException;
import java.security.KeyPair;
import java.security.NoSuchProviderException;
import java.security.cert.CRLException;
import java.security.cert.CertificateException;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.List;


// TODO: 2019/2/21 签名运算后期需要调用加密机
public class PkixIssueReqBuilder extends BasicPkixIssueBuilder {

    private String hashAlgorithm;

    private Logger logger = LoggerFactory.getLogger(PkixIssueReqBuilder.class);

    public PkixIssueReqBuilder(KeyPair keyPair, X509Certificate certificate, String hashAlgorithm) {
        super(keyPair, certificate);
        this.hashAlgorithm = hashAlgorithm;
    }
    public PkixIssueReqBuilder(int privateKeyIndex, String privateKeyPassword, X509Certificate certificate,
                               String hashAlgorithm, SdfCryptoType sdfCryptoType) {
        super(privateKeyIndex, privateKeyPassword,certificate, sdfCryptoType);
        this.hashAlgorithm = hashAlgorithm;
    }
    public PkixIssueReq build(X509Certificate oldWithNew, X509Certificate newWithOld, X509Certificate newWithNew) throws Exception {
        if (X509Utils.verifyCert(newWithOld, this.certificate.getPublicKey())
                && X509Utils.verifyCert(newWithNew, newWithOld.getPublicKey())
                && X509Utils.verifyCert(oldWithNew, newWithNew.getPublicKey())) {
            this.certificate = newWithNew;
            return generate(TBSIssueType.UPDATE_ROOT_CERTIFICATE, new Certificate[]{X509Utils.convertCertificate(oldWithNew),
                    X509Utils.convertCertificate(newWithOld), X509Utils.convertCertificate(newWithNew)}, null, null);
        }
        throw new Exception("signature is not valid");
    }


    public PkixIssueReq build(X509Certificate cert) throws Exception {
        if (X509Utils.verifyCert(cert, this.certificate.getPublicKey())) {
            return generate(TBSIssueType.SEND_CERTIFICATE, new Certificate[]{X509Utils.convertCertificate(cert)}, null, null);
        }
        throw new Exception("signature is not valid");
    }

    public PkixIssueReq buildSendCerts(List<X509Certificate> certs) throws Exception {
        Certificate[] certificates = new Certificate[certs.size()];
        for (int i = 0; i < certs.size(); i++) {
            if (X509Utils.verifyCert(certs.get(i), this.certificate.getPublicKey())) {
                certificates[i] = X509Utils.convertCertificate(certs.get(i));
            } else {
                throw new Exception("index in " + i + " cert signature is not valid");
            }
        }
        return generate(TBSIssueType.SEND_CERTIFICATE, certificates, null, null);
    }

    public PkixIssueReq build(int crlSegment, X509CRL crl) throws Exception {
        if (X509Utils.verifyCRL(crl, this.certificate.getPublicKey())) {
            return generate(TBSIssueType.SEND_CRL, null, null, new ASN1CRL[]{generateASN1CRL(crlSegment, crl)});
        }
        throw new Exception("signature is not valid");
    }

    public PkixIssueReq buildSendCrls(List<PkixIssueCRL> list) throws Exception {
        ASN1CRL[] asn1CRLS = new ASN1CRL[list.size()];
        for (int i = 0; i < list.size(); i++) {
            if (X509Utils.verifyCRL(list.get(i).getCrl(), this.certificate.getPublicKey())) {
                asn1CRLS[i] = generateASN1CRL(list.get(i).getCrlSegment(), list.get(i).getCrl());
            } else {
                throw new Exception("index in " + i + " crl signature is not valid");
            }

        }
        return generate(TBSIssueType.SEND_CRL, null, null, asn1CRLS);
    }

    public PkixIssueReq build(TBSIssueCRLReason reason, X509Certificate certificate) throws Exception {
        if (X509Utils.verifyCert(certificate, this.certificate.getPublicKey())) {
            return generate(TBSIssueType.SEND_CERTIFICATE_STATUS, null, new CertStatus[]{generateCertStatus(reason.getType(),
                    certificate)}, null);
        }
        throw new Exception("signature is not valid");
    }

    public PkixIssueReq buildSendCertStatuses(List<PkixIssueCertStatus> list) throws Exception {
        CertStatus[] statuses = new CertStatus[list.size()];
        for (int i = 0; i < list.size(); i++) {
            if (X509Utils.verifyCert(list.get(i).getCertificate(), this.certificate.getPublicKey())) {
                statuses[i] = generateCertStatus(list.get(i).getReason().getType(), list.get(i).getCertificate());
            } else {
                throw new Exception("index in " + i + " cert signature is not valid");
            }
        }
        return generate(TBSIssueType.SEND_CERTIFICATE_STATUS, null, statuses, null);
    }

    // TODO: 2019/4/1 交叉证书forward dn与当前证书dn相同时候 无法验证
    public PkixIssueReq build(X509CertificatePair pair) throws Exception {
        if (pair.getForward() == null && pair.getReverse() == null) {
            throw new Exception("can not be all null");
        }
        //reserve先验公钥和CA是否相同 证明非自签 不同继续 在进行验签证明
        if (pair.getForward() == null) {
            if (X509Utils.verifyCert(pair.getReverse(), this.certificate.getPublicKey())
                    && !pair.getReverse().getPublicKey().equals(this.certificate.getPublicKey())) {
                return generate(TBSIssueType.SEND_CROSS_CERTIFICATE, new Certificate[]{
                        X509Utils.convertCertificate(pair.getReverse())
                }, null, null);
            }
            throw new Exception("signature is not valid");
        }
        if (pair.getReverse() == null) {
            //forward先验公钥和CA是否相同  在进行验签证明非自签
            if (pair.getForward().getPublicKey().equals(this.certificate.getPublicKey())
                    && !X509Utils.verifyCert(pair.getForward(), this.certificate.getPublicKey())) {
                return generate(TBSIssueType.SEND_CROSS_CERTIFICATE, new Certificate[]{
                        X509Utils.convertCertificate(pair.getForward())
                }, null, null);
            }
            throw new Exception("signature is not valid");
        }
        if (X509Utils.verifyCert(pair.getReverse(), this.certificate.getPublicKey())
                && !pair.getReverse().getPublicKey().equals(this.certificate.getPublicKey())
                && pair.getForward().getPublicKey().equals(this.certificate.getPublicKey())
                && !X509Utils.verifyCert(pair.getForward(), this.certificate.getPublicKey())) {
            return generate(TBSIssueType.SEND_CROSS_CERTIFICATE, new Certificate[]{
                    X509Utils.convertCertificate(pair.getForward()),
                    X509Utils.convertCertificate(pair.getReverse())
            }, null, null);
        }
        throw new Exception("signature is not valid");

    }

    private ASN1CRL generateASN1CRL(int crlSegment, X509CRL crl) throws CRLException, CertificateException, NoSuchProviderException, IOException {
        ASN1Integer stateOrProvinceNum = new ASN1Integer(0);
        ASN1Integer crlSegmentASN1 = new ASN1Integer(crlSegment);
        ASN1Integer cRLNumber = new ASN1Integer(crl.getExtensionValue(Extension.cRLNumber.getId()));
        ASN1Integer deltareCRLNumber = null;
        if (X509Utils.isDRL(crl)) {
            byte[] bytes = crl.getExtensionValue(Extension.deltaCRLIndicator.getId());
            if (bytes != null) {
                deltareCRLNumber = ASN1Integer.getInstance(DEROctetString.getInstance(bytes).getOctets());
            }
        }

        CertificateList certificateList = CertificateList.getInstance(crl.getEncoded());

        return new ASN1CRL(stateOrProvinceNum, crlSegmentASN1, cRLNumber, deltareCRLNumber, certificateList);
    }

    private CertStatus generateCertStatus(int reason, X509Certificate cert) throws Exception {
        // TODO: 2019/3/27 hash是否使用密码机 （可不使用）
        CertificateID certId = new CertificateID((new BcDigestCalculatorProvider())
                .get(new DefaultDigestAlgorithmIdentifierFinder().find(this.hashAlgorithm)),
                new X509CertificateHolder(this.certificate.getEncoded()), cert.getSerialNumber());

        ASN1GeneralizedTime beforeTime = new ASN1GeneralizedTime(cert.getNotBefore());
        ASN1GeneralizedTime endTime = new ASN1GeneralizedTime(cert.getNotAfter());
        //证书状态
        ASN1Integer asn1Status = TBSIssueCRLStatus.INVALID.encode();
        ASN1GeneralizedTime statusTime = new ASN1GeneralizedTime(new Date(System.currentTimeMillis()));
        CRLReason crlReason = CRLReason.lookup(reason);

        return new CertStatus(certId.toASN1Primitive(), beforeTime, endTime, asn1Status, statusTime, crlReason);
    }

    private PkixIssueReq generate(TBSIssueType type, Certificate[] cert, CertStatus[] certStatus, ASN1CRL[] mCRL) throws Exception {
        ASN1Integer typeASN1 = type.encode();
        ASN1OctetString transNonce = PkixIssueUtils.generateTransNonce();
        ASN1GeneralizedTime time = PkixIssueUtils.generateTime();
        ASN1Integer number;
        switch (type) {
            case UPDATE_ROOT_CERTIFICATE:
            case SEND_CERTIFICATE:
            case SEND_CROSS_CERTIFICATE:
                number = new ASN1Integer(cert.length);
                break;
            case SEND_CRL:
                number = new ASN1Integer(mCRL.length);
                break;
            case SEND_CERTIFICATE_STATUS:
                number = new ASN1Integer(certStatus.length);
                break;
            default:
                throw new Exception("unknown type " + type);
        }

        TBSIssueRequest pkixIssueInfo = new TBSIssueRequest(typeASN1, transNonce, number, time, cert, certStatus, mCRL);
        Signature signature = this.getSignature(pkixIssueInfo);
        return new PkixIssueReq(pkixIssueInfo, signature);
    }
}
